#include "graphics.h"
#include <math.h>

CGraphics *CGraphics::Instance = NULL;

CGraphics *CGraphics::instance()
{
    if(Instance == NULL)
    {
        Instance = new CGraphics();
    }
    return Instance;
}

CGraphics::CGraphics()
{
    db = create_bitmap(800, 600); //make the double buffer
    clear_to_color(db, makecol(0, 0, 0));
    effectOffset = 50;
    
    DATAFILE *guiDat = load_datafile("data/gui.dat");
    if(guiDat == NULL)
    {
        allegro_message("Failed to load gui datfile");
        ebar = NULL;
    }
    else
    {
        ebar = create_bitmap(((BITMAP *)guiDat[0].dat)->w, ((BITMAP *)guiDat[0].dat)->h);
        blit((BITMAP *)guiDat[0].dat, ebar, 0, 0, 0, 0, ebar->w, ebar->h);
        unload_datafile(guiDat);
        BITMAP *temp = create_bitmap(800, 600);
        back = create_bitmap(800, 600);
        clear_to_color(back, makecol(0, 0, 0));
        for(int i = 0; i < 256; i++)
            line(temp, 0, 300 - i, 800, 300 - i, makecol(255 - i, 0, 0));
        floodfill(temp, 0, 399, makecol(255, 0, 0));
        for(int i = 0; i < 800; i++)
            blit(temp, back, i, 0, i, 300 + (int)(30 * sin((double)(i + effectOffset) * (double)0.0174532925)), 1, 600);
        destroy_bitmap(temp);
        
    }
}

CGraphics::~CGraphics()
{
    destroy_bitmap(db);
    destroy_bitmap(ebar);
    destroy_bitmap(back);
}

void CGraphics::update()
{
    effectOffset++;
    while(effectOffset > 360)
        effectOffset -= 360;
}

void CGraphics::drawDB(int mode, int life)
{
    //Draw HUD first
    if(life >= 0)
    {
        draw_sprite(db, ebar, 0, 0);
        rectfill(db, 9, 34, 21, 336, makecol(255, 255, 255));
        rectfill(db, 10, 335 - life * 3, 20, 335, makecol((int)(255 * sin((double)life * 0.9 * 0.0174532925)), 0, (int)(255 * sin((double)(100 - life) * 0.9 * 0.0174532925)))); //sin color changing
        textout_ex(db, font, "Energy", 0, -4, makecol(255, 255, 255), -1);
    }
    //now blit it according to draw style
    acquire_screen();
    if(mode == drawNormal)
    {
        blit(db, screen, 0, 0, 0, 0, 800, 600);
    }
    else if(mode == drawDrunk)
    {
        BITMAP *tb = create_bitmap(800 + 80, 600);
        for(int i = 0; i < 600; i++)
        {
            blit(db, tb, 0, i, (int)(30 * sin((double)(i + effectOffset) * (double)0.0174532925)), i, 800, 1);
        }
        blit(tb, screen, 0, 0, 0, 0, 800, 600);
        destroy_bitmap(tb);
    }
    else if(mode == drawScrewed)
    {
        BITMAP *tb = create_bitmap(800, 600);
        BITMAP *qb = create_bitmap(800, 600);
        for(int i = 0; i < 600; i++)
            blit(db, tb, 0, i, (int)(40 * sin((double)(i + effectOffset) * (double)0.0174532925)), i, 800, 1);
        for(int i = 0; i < 800; i++)
            blit(tb, qb, i, 0, i, (int)(40 * sin((double)(i + effectOffset) * (double)0.0174532925)), 1, 600);
        blit(qb, screen, 0, 0, 0, 0, 800, 600);
        destroy_bitmap(tb);
        destroy_bitmap(qb);
    }
    release_screen();
    blit(back, db, 0, 0, 0, 0, 800, 600);
}
