/*
Written By: Ben Goska [IcedRazor]
For: TINS 2005
*/

#include<allegro.h>
#ifdef WIN32
    #include<winalleg.h>
#endif
#include<list>
#include<fstream>
using namespace std;

#include "level.h"
#include "graphics.h"
#include "animation.h"
#include "player.h"
#include "enemy.h"

enum {stateInit, stateSplash, stateWin, stateStory, stateLose, stateMenu, stateInGame, stateNextLevel, stateFinished};

int logicTimerVar = 0;

void logicTimerFunc()
{
    logicTimerVar++;
}

int init()
{
    allegro_init();
    override_config_file("config.cfg");
    int depth = get_config_int(NULL, "color_depth", 32);
    int fullscreen = get_config_int(NULL, "fullscreen", 1);
    
    if(depth == -1)
        depth = desktop_color_depth();
    if(depth == 0)
        depth = 16;
        
    if(fullscreen)
        fullscreen = GFX_AUTODETECT_FULLSCREEN;
    else
        fullscreen = GFX_AUTODETECT_WINDOWED;
    /*
    if you have a slow computer change the color depth, this will only matter a lot when playing in drunk or screwed mode
        CPU > 1ghz  use 32/24
        CPU < 1ghz  use 16/15
    don't use 8bit, makes it all messed up because of masking
    */
    set_color_depth(depth);
    
    if(set_gfx_mode(fullscreen, 800, 600, 0, 0))
    {
        allegro_message("GFX Error\n");
        return 1;
    }
    set_window_title("Tollus - TINS 2005 entry [IcedRazor]");
    install_keyboard();
    install_mouse();
    install_timer();
    LOCK_VARIABLE(logicTimerVar);
    LOCK_FUNCTION(logicTimerFunc);
    install_int_ex(logicTimerFunc, BPS_TO_TIMER(60));
    return 0;
}

int main()
{
    if(init())
        return 1;
    bool quitNow = false;
    
    //UNLOCK ABLE STUFF, SWITCH TO TRUE TO UNLOCK THAT MODE
    bool drunkUnlocked = false;
    bool screwedUnlocked = false;
    
    ifstream cheatfile("iamacheater");
    if(cheatfile.is_open()) //if the cheat file exists enable everything
    {
        drunkUnlocked = true;
        screwedUnlocked = true;
        cheatfile.close();
    }
    
    int state = stateInit;
    int level = 0;
    int timeRemaining = 0;
    int displayMode = drawNormal;
    bool antiBunnyHop = false; //aids in removing the ability to bunny hop (its just plain cheating in this game)
    DATAFILE *levelData = NULL;
    DATAFILE *playerData = NULL;
    DATAFILE *enemyData = NULL;
    DATAFILE *fontData = NULL;
    DATAFILE *guidat = NULL;
    FONT *smallFont = NULL;
    BITMAP *tbar = NULL;
    Clevel *currentLevel = NULL;
    CPlayer *player = NULL;
    while(!quitNow)
    {
        switch(state)
        {
            case stateInit:
                //load bitmaps/datafiles for menu, game, everything
                levelData = load_datafile("data/level.dat");
                playerData = load_datafile("data/samus.dat");
                enemyData = load_datafile("data/ring.dat");
                fontData = load_datafile("data/font.dat");
                guidat = load_datafile("data/gui.dat");
                if( levelData == NULL || playerData == NULL || enemyData == NULL || fontData == NULL || guidat == NULL)
                {
                    allegro_message("Datafile loading error!\n");
                    state = stateFinished;
                }
                else
                {
                    player = new CPlayer(playerData);
                    smallFont = font;
                    font = (FONT *)fontData[0].dat;
                    tbar = create_bitmap(((BITMAP *)guidat[1].dat)->w, ((BITMAP *)guidat[1].dat)->h);
                    blit((BITMAP *)guidat[1].dat, tbar, 0, 0, 0, 0, tbar->w, tbar->h);
                    unload_datafile(guidat);
                    CEnemy::init(enemyData);
                    state = stateMenu;
                }
                break;
            case stateStory:
                clear_keybuf(); //I should have used some sort of auto line break system
                textprintf_ex(CGraphics::instance()->getDB(), font, 2, 0, makecol(255, 255, 255), -1,    "    It was a quiet day, Samus decided to");
                textprintf_ex(CGraphics::instance()->getDB(), font, 2, 30, makecol(255, 255, 255), -1,   "stop hunting and take a nap. As she slept");
                textprintf_ex(CGraphics::instance()->getDB(), font, 2, 60, makecol(255, 255, 255), -1,   "a troll came by. Seeing that she was asleep");
                textprintf_ex(CGraphics::instance()->getDB(), font, 2, 90, makecol(255, 255, 255), -1,   "he decided that it would be fun to take her");
                textprintf_ex(CGraphics::instance()->getDB(), font, 2, 120, makecol(255, 255, 255), -1,  "suit. The troll took the gravity suit from");
                textprintf_ex(CGraphics::instance()->getDB(), font, 2, 150, makecol(255, 255, 255), -1,  "Samus. When the troll got into the suit he");
                textprintf_ex(CGraphics::instance()->getDB(), font, 2, 180, makecol(255, 255, 255), -1,  "broke everything, including the guns.");
                textprintf_ex(CGraphics::instance()->getDB(), font, 2, 240, makecol(255, 255, 255), -1,  "    Too bad for the troll, mother brain");
                textprintf_ex(CGraphics::instance()->getDB(), font, 2, 270, makecol(255, 255, 255), -1,  "tracked the suit and trapped the troll");
                textprintf_ex(CGraphics::instance()->getDB(), font, 2, 300, makecol(255, 255, 255), -1,  "thinking that it was Samus. Then mother");
                textprintf_ex(CGraphics::instance()->getDB(), font, 2, 330, makecol(255, 255, 255), -1,  "brain filled the room the troll was trapped");
                textprintf_ex(CGraphics::instance()->getDB(), font, 2, 360, makecol(255, 255, 255), -1,  "in with the gravity suit's only weakness,");
                textprintf_ex(CGraphics::instance()->getDB(), font, 2, 390, makecol(255, 255, 255), -1,  "flaming cherios of death!");
                textprintf_ex(CGraphics::instance()->getDB(), font, 2, 440, makecol(255, 255, 255), -1,  "    Help the troll live as long as possible");
                textprintf_ex(CGraphics::instance()->getDB(), font, 2, 470, makecol(255, 255, 255), -1,  "by jumping and avoiding the cherios.");
                textprintf_ex(CGraphics::instance()->getDB(), font, 2, 530, makecol(255, 255, 255), -1,  "[press enter to continue]");
                CGraphics::instance()->drawDB(drawNormal, -1);
                if(keypressed())
                    state = stateMenu;
                clear_keybuf();
                break;
            case stateSplash:
                textprintf_centre_ex(CGraphics::instance()->getDB(), font, 400, 100, makecol(255, 255, 255), -1, "You are on level: %i", level);
                textprintf_centre_ex(CGraphics::instance()->getDB(), font, 400, 140, makecol(255, 255, 255), -1, "[press enter to continue]");
                CGraphics::instance()->drawDB(drawNormal, -1);
                if(key[KEY_ENTER])
                    state = stateInGame;
                logicTimerVar = 0;
                break;
            case stateWin:
                textprintf_centre_ex(CGraphics::instance()->getDB(), font, 400, 100, makecol(255, 255, 255), -1, "You beat all ten levels");
                if(!screwedUnlocked || !drunkUnlocked)
                    textprintf_centre_ex(CGraphics::instance()->getDB(), font, 400, 140, makecol(255, 255, 255), -1, "something was unlocked");
                textprintf_centre_ex(CGraphics::instance()->getDB(), font, 400, 180, makecol(255, 255, 255), -1, "[press enter to continue]");
                CGraphics::instance()->drawDB(drawNormal, -1);
                if(key[KEY_ENTER])
                {
                    level = 0;
                    screwedUnlocked = true;
                    drunkUnlocked = true;
                    state = stateMenu;
                }
                logicTimerVar = 0;
                break;
            case stateMenu:
                //display menu and such
                textprintf_centre_ex(CGraphics::instance()->getDB(), font, 400, 200, makecol(180, 180, 180), -1, "Play Game");
                textprintf_centre_ex(CGraphics::instance()->getDB(), font, 400, 240, makecol(180, 180, 180), -1, "View Story");
                textprintf_centre_ex(CGraphics::instance()->getDB(), font, 400, 280, makecol(180, 180, 180), -1, "Quit");
                
                textprintf_ex(CGraphics::instance()->getDB(), smallFont, 0, 560, makecol(255, 255, 255), -1, "Sprites ripped by: Tommy Lee");
                textprintf_ex(CGraphics::instance()->getDB(), smallFont, 0, 570, makecol(255, 255, 255), -1, "Tiles ripped by: Xitus");
                textprintf_ex(CGraphics::instance()->getDB(), smallFont, 0, 580, makecol(255, 255, 255), -1, "Sprites/Tiles made/owned by: Nintendo");
                textprintf_ex(CGraphics::instance()->getDB(), smallFont, 0, 590, makecol(255, 255, 255), -1, "Thanks guys!");
                textprintf_right_ex(CGraphics::instance()->getDB(), smallFont, 800, 590, makecol(255, 255, 255), -1, "Game By: Ben Goska [IcedRazor]");
                
                if(drunkUnlocked)
                    textprintf_centre_ex(CGraphics::instance()->getDB(), font, 400, 340, makecol(180, 180, 180), -1, "Play Drunk");
                if(screwedUnlocked)
                    textprintf_centre_ex(CGraphics::instance()->getDB(), font, 400, 380, makecol(180, 180, 180), -1, "Play Screwed");
                show_mouse(CGraphics::instance()->getDB());
                if(mouse_b) //if something is pressed
                {
                    int x = mouse_x;
                    int y = mouse_y;
                    if((x > 310)&&(x < 495)&&(y > 200)&&(y < 230))
                    {
                        //play
                        displayMode = drawNormal;
                        state = stateNextLevel;
                    }
                    else if((x > 305)&&(x < 500)&&(y > 240)&&(y < 275))
                    {
                        //view
                        state = stateStory;
                    }
                    else if((x > 360)&&(x < 440)&&(y > 285)&&(y < 310))
                    {
                        //quit
                        state = stateFinished;
                    }
                    else if((x > 300)&&(x < 500)&&(y > 340)&&(y < 375)&&(drunkUnlocked))
                    {
                        //play d
                        displayMode = drawDrunk;
                        state = stateNextLevel;
                    }
                    else if((x > 285)&&(x < 515)&&(y > 385)&&(y < 410)&&(screwedUnlocked))
                    {
                        //play s
                        displayMode = drawScrewed;
                        state = stateNextLevel;
                    }
                }
                else
                {
                    int x = mouse_x;
                    int y = mouse_y;
                    if((x > 310)&&(x < 495)&&(y > 200)&&(y < 230))
                    {
                        //play
                        textprintf_centre_ex(CGraphics::instance()->getDB(), font, 400, 200, makecol(255, 255, 255), -1, "Play Game");
                    }
                    else if((x > 305)&&(x < 500)&&(y > 240)&&(y < 275))
                    {
                        //view
                        textprintf_centre_ex(CGraphics::instance()->getDB(), font, 400, 240, makecol(255, 255, 255), -1, "View Story");
                    }
                    else if((x > 360)&&(x < 440)&&(y > 285)&&(y < 310))
                    {
                        //quit
                        textprintf_centre_ex(CGraphics::instance()->getDB(), font, 400, 280, makecol(255, 255, 255), -1, "Quit");
                    }
                    else if((x > 300)&&(x < 500)&&(y > 340)&&(y < 375)&&(drunkUnlocked))
                    {
                        //play d
                        textprintf_centre_ex(CGraphics::instance()->getDB(), font, 400, 340, makecol(255, 255, 255), -1, "Play Drunk");
                    }
                    else if((x > 285)&&(x < 515)&&(y > 385)&&(y < 410)&&(screwedUnlocked))
                    {
                        //play s
                        textprintf_centre_ex(CGraphics::instance()->getDB(), font, 400, 380, makecol(255, 255, 255), -1, "Play Screwed");
                    }
                }
                CGraphics::instance()->drawDB(drawNormal, -1);
                show_mouse(NULL);
                break;
            case stateLose:
                textprintf_centre_ex(CGraphics::instance()->getDB(), font, 400, 100, makecol(255, 0, 0), -1, "You died");
                textprintf_centre_ex(CGraphics::instance()->getDB(), font, 400, 140, makecol(255, 255, 255), -1, "You got to level %i, not bad", level);
                textprintf_centre_ex(CGraphics::instance()->getDB(), font, 400, 180, makecol(255, 255, 255), -1, "[press enter to continue]");
                if(level <= 4)
                    textprintf_centre_ex(CGraphics::instance()->getDB(), font, 400, 240, makecol(180, 180, 180), -1, "Beat level 4 to unlock stuff");
                if((level > 4)&&(!drunkUnlocked))
                {
                    textprintf_centre_ex(CGraphics::instance()->getDB(), font, 400, 240, makecol(180, 180, 180), -1, "Something was unlocked");
                }
                CGraphics::instance()->drawDB(drawNormal, -1);
                if(key[KEY_ENTER])
                {
                    state = stateMenu;
                    if(level > 4)
                        drunkUnlocked = true;
                    level = 0;
                }
                logicTimerVar = 0;
                break;
            case stateInGame:
                //play current level
                if(logicTimerVar)
                {
                    if(key[KEY_ESC])
                    {
                        state = stateMenu;
                        level--;
                    }
                    CGraphics::instance()->update();
                    CEnemy::instance()->update(player);
                    int input = 0;
                    if(key[KEY_LEFT])
                        input += inputLeft;
                    if(key[KEY_RIGHT])
                        input += inputRight;
                    if(key[KEY_SPACE])
                    {
                        if(!antiBunnyHop)
                        {
                            input += inputJump;
                            antiBunnyHop = true;
                        }
                    }
                    else
                    {
                        antiBunnyHop = false;
                    }
                    player->update(input, currentLevel);
                    CEnemy::instance()->update(player);
                    if(timeRemaining <= 0)
                    {
                        state = stateNextLevel;
                    }
                    if(player->getLife() <= 0) //loser
                    {
                        state = stateLose;
                    }
                    timeRemaining--;
                    logicTimerVar--;
                }
                else
                {
                    currentLevel->draw();
                    player->draw();
                    CEnemy::instance()->draw();
                    //draw time remaining
                    draw_sprite(CGraphics::instance()->getDB(), tbar, 800 - tbar->w, 0);
                    textprintf_right_ex(CGraphics::instance()->getDB(), font, 800, -4, makecol(255, 255, 255), -1, "%i secconds left", (int)(timeRemaining / 60));
                    CGraphics::instance()->drawDB(displayMode, player->getLife());
                }
                break;
            case stateNextLevel:
                //load next level
                level++;
                if(currentLevel != NULL)
                    delete currentLevel;
                if(player != NULL)
                    delete player;
                player = new CPlayer(playerData);
                currentLevel = new Clevel(24, 19, 10 + 5 * level, level, levelData);
                currentLevel->setToStart(player);
                CEnemy::instance()->setNumber(8 + 4 * level, currentLevel);
                timeRemaining = (level * 5 + 10) * 60; //level * 15 secconds
                if(level > 10)
                {
                    state = stateWin;
                }
                else
                    state = stateSplash;
                break;
            case stateFinished:
                //destroy everything
                if(currentLevel != NULL)
                    delete currentLevel;
                if(levelData != NULL)
                    unload_datafile(levelData);
                if(playerData != NULL)
                    unload_datafile(playerData);
                if(enemyData != NULL)
                    unload_datafile(enemyData);
                if(player != NULL)
                    delete player;
                if(fontData != NULL)
                    unload_datafile(fontData);
                quitNow = true;
                CGraphics::instance()->~CGraphics(); //kill singletons
                CEnemy::instance()->~CEnemy(); 
                break;
        }
    }
    return 0;
}
END_OF_MAIN()
